package types

import (
	"encoding/json"
	"fmt"
	"io"
	"time"
)

// Device ...
type Device struct {
	DeviceID          string    `json:"device_id"`
	DeviceName        *string   `json:"device_name"`
	DeviceLocation    *string   `json:"device_location"`
	DeviceLastContact time.Time `json:"device_last_contact"`
	CreationDate      time.Time `json:"creation_date"`
}

// EncodeToJSON needs a writer to write the struct into json string
func (d *Device) EncodeToJSON(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", "  ")
	err := encoder.Encode(&d)
	if err != nil {
		return fmt.Errorf("Error in encoding struct to json: %v", err)
	}
	return nil
}

// DecodeFromJSON decode a json string from a reader into a struct
func (d *Device) DecodeFromJSON(r io.Reader) error {
	decoder := json.NewDecoder(r)
	if err := decoder.Decode(&d); err != nil {
		return fmt.Errorf("Can not unmarshal JSON: %v", err)
	}
	return nil
}

// Failnoise ...
type Failnoise struct {
	FailnoiseID        string    `json:"failnoise_id"`
	FailnoiseCounter   int       `json:"failnoise_counter,string"`
	FailnoiseStartDate time.Time `json:"failnoise_start_date"`
	FailnoiseEndDate   time.Time `json:"failnoise_end_date"`
	SensorID           string    `json:"sensor_id"`
	CreationDate       time.Time `json:"creation_date"`
}

// EncodeToJSON needs a writer to write the struct into json string
func (f *Failnoise) EncodeToJSON(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", "  ")
	err := encoder.Encode(&f)
	if err != nil {
		return fmt.Errorf("Error in encoding struct to json: %v", err)
	}
	return nil
}

// DecodeFromJSON decode a json string from a reader into a struct
func (f *Failnoise) DecodeFromJSON(r io.Reader) error {
	decoder := json.NewDecoder(r)
	if err := decoder.Decode(&f); err != nil {
		return fmt.Errorf("Can not unmarshal JSON: %v", err)
	}
	return nil
}

// Humidity ...
type Humidity struct {
	HumidityID    string    `json:"humidity_id"`
	HumidityValue float64   `json:"humidity_value,string"`
	HumidityDate  time.Time `json:"humidity_date"`
	SensorID      string    `json:"sensor_id"`
	CreationDate  time.Time `json:"creation_date"`
}

// EncodeToJSON needs a writer to write the struct into json string
func (h *Humidity) EncodeToJSON(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", "  ")
	err := encoder.Encode(&h)
	if err != nil {
		return fmt.Errorf("Error in encoding struct to json: %v", err)
	}
	return nil
}

// DecodeFromJSON decode a json string from a reader into a struct
func (h *Humidity) DecodeFromJSON(r io.Reader) error {
	decoder := json.NewDecoder(r)
	if err := decoder.Decode(&h); err != nil {
		return fmt.Errorf("Can not unmarshal JSON: %v", err)
	}
	return nil
}

// Knock ...
type Knock struct {
	KnockID        string    `json:"knock_id"`
	KnockCounter   int       `json:"knock_counter,string"`
	KnockStartDate time.Time `json:"knock_start_date"`
	KnockEndDate   time.Time `json:"knock_end_date"`
	SensorID       string    `json:"sensor_id"`
	CreationDate   time.Time `json:"creation_date"`
}

// EncodeToJSON needs a writer to write the struct into json string
func (k *Knock) EncodeToJSON(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", "  ")
	err := encoder.Encode(&k)
	if err != nil {
		return fmt.Errorf("Error in encoding struct to json: %v", err)
	}
	return nil
}

// DecodeFromJSON decode a json string from a reader into a struct
func (k *Knock) DecodeFromJSON(r io.Reader) error {
	decoder := json.NewDecoder(r)
	if err := decoder.Decode(&k); err != nil {
		return fmt.Errorf("Can not unmarshal JSON: %v", err)
	}
	return nil
}

// Sensor ...
type Sensor struct {
	SensorID          string    `json:"sensor_id"`
	SensorName        *string   `json:"sensor_name"`
	SensorLocation    *string   `json:"sensor_location"`
	WireID            *string   `json:"wire_id"`
	GPIONumber        *string   `json:"gpio_number"`
	SensorType        *string   `json:"sensor_type"`
	SensorEnabled     *bool     `json:"sensor_enabled"`
	SensorLastContact time.Time `json:"sensor_last_contact"`
	DeviceID          string    `json:"device_id"`
	CreationDate      time.Time `json:"creation_date"`
}

// EncodeToJSON needs a writer to write the struct into json string
func (s *Sensor) EncodeToJSON(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", "  ")
	err := encoder.Encode(&s)
	if err != nil {
		return fmt.Errorf("Error in encoding struct to json: %v", err)
	}
	return nil
}

// DecodeFromJSON decode a json string from a reader into a struct
func (s *Sensor) DecodeFromJSON(r io.Reader) error {
	decoder := json.NewDecoder(r)
	if err := decoder.Decode(&s); err != nil {
		return fmt.Errorf("Can not unmarshal JSON: %v", err)
	}
	return nil
}

// ReadableSensorName returns a readable sensor name
func (s *Sensor) ReadableSensorName() string {
	if s.SensorName != nil {
		return *s.SensorName
	}
	if s.WireID != nil {
		return *s.WireID
	}
	return s.SensorID
}

// Temperature ...
type Temperature struct {
	TemperatureID    string    `json:"temperature_id"`
	TemperatureValue float64   `json:"temperature_value,string"`
	TemperatureDate  time.Time `json:"temperature_date"`
	SensorID         string    `json:"sensor_id"`
	CreationDate     time.Time `json:"creation_date"`
}

// EncodeToJSON needs a writer to write the struct into json string
func (t *Temperature) EncodeToJSON(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", "  ")
	err := encoder.Encode(&t)
	if err != nil {
		return fmt.Errorf("Error in encoding struct to json: %v", err)
	}
	return nil
}

// DecodeFromJSON decode a json string from a reader into a struct
func (t *Temperature) DecodeFromJSON(r io.Reader) error {
	decoder := json.NewDecoder(r)
	if err := decoder.Decode(&t); err != nil {
		return fmt.Errorf("Can not unmarshal JSON: %v", err)
	}
	return nil
}

// Throughput ...
type Throughput struct {
	ThroughputID        string    `json:"throughput_id"`
	ThroughputAverage   float64   `json:"throughput_average,string"`
	ThroughputStartDate time.Time `json:"throughput_start_date"`
	ThroughputEndDate   time.Time `json:"throughput_end_date"`
	SensorID            string    `json:"sensor_id"`
	CreationDate        time.Time `json:"creation_date"`
}

// EncodeToJSON needs a writer to write the struct into json string
func (t *Throughput) EncodeToJSON(w io.Writer) error {
	encoder := json.NewEncoder(w)
	encoder.SetIndent("", "  ")
	err := encoder.Encode(&t)
	if err != nil {
		return fmt.Errorf("Error in encoding struct to json: %v", err)
	}
	return nil
}

// DecodeFromJSON decode a json string from a reader into a struct
func (t *Throughput) DecodeFromJSON(r io.Reader) error {
	decoder := json.NewDecoder(r)
	if err := decoder.Decode(&t); err != nil {
		return fmt.Errorf("Can not unmarshal JSON: %v", err)
	}
	return nil
}
